/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gio/gio.h>

#include "uat-common.h"

extern ua_device_h g_device_h; /**< Device handle */
extern ua_user_h g_user_h; /**< User handle */
ua_service_h g_service_h = NULL; /**< Service handle */
char g_service_str[MENU_DATA_SIZE + 1] = { "ua.service.default" }; /**< Seletected service name */

extern char g_user_account_str[MENU_DATA_SIZE + 1]; /**< User account */
extern char g_user_name_str[MENU_DATA_SIZE + 1]; /**< User name */

static char g_presence_threshold_str[MENU_DATA_SIZE + 1] = {0,}; /**< Seletected service presence threshold */
static char g_absence_threshold_str[MENU_DATA_SIZE + 1] = {0,}; /**< Seletected service absence threshold */
static char g_selected_service_id[MENU_DATA_SIZE + 1] = {0,}; /**< Selected service id in the list */
static char g_service_device_discriminant[MENU_DATA_SIZE + 1] = {"1",}; /**< Discriminant for a device in a service */

extern char g_selected_device_idx[MENU_DATA_SIZE + 1]; /**< Selected device index */
extern GSList *g_device_list; /**< Device List */
extern int g_device_count; /**< Seletected service name */

extern char g_selected_user_idx[MENU_DATA_SIZE + 1]; /**< Selected user index */
extern GSList *user_list; /**< User List */
extern int idx_user; /**< Found device sequence number */

static GSList *g_service_list = NULL; /**< Service List */
static int g_service_count = 0; /**< Service count */

static bool _service_foreach_added_user_cb(
	ua_service_h service_handle, ua_user_h user_handle, void *user_data)
{
	int ret = UA_ERROR_NONE;

	ua_user_h handle = NULL;
	char *service_name = NULL;
	char *account = NULL;
	char *name = NULL;
	unsigned long long last_seen = 0;

	ret = ua_service_get_name(service_handle, &service_name);
	if (UA_ERROR_NONE == ret)
		msgp("Cur. Service name : %s\n", service_name);

	g_free(service_name);

	if (!user_handle) {
		msgr("user_handle is NULL");
		return true;
	}

	ret = ua_user_clone(&handle, user_handle);
	if (UA_ERROR_NONE != ret) {
		msgr("ua_user_clone() result: [0x%X] [%s]",
			ret, uat_get_error_str(ret));
	}

	user_list = g_slist_append(user_list, handle);
	idx_user++;

	ret = ua_user_get_account(handle, &account);
	if (UA_ERROR_NONE == ret) {
		msglr("[%d]", idx_user);
		msglr("User account info %s", account);
		g_free(account);
	}

	ret = ua_user_get_name(handle, &name);
	if (UA_ERROR_NONE == ret) {
		msglr("User Name : %s", name);
		g_free(name);
	}

	ret = ua_user_get_last_presence(handle, &last_seen);
	if (UA_ERROR_NONE == ret)
		msglr("last present at %llu", last_seen);


	return true;
}

static void __update_service_info(void)
{
	int ret = UA_ERROR_NONE;
	char *name = NULL;

	if (1 == g_service_count) {
		if (g_service_h) {
			ua_service_destroy(g_service_h);
			g_service_h = NULL;
		}

		ua_service_get_default_service(&g_service_h);
	}

	ret = ua_service_get_name(g_service_h, &name);
	if (UA_ERROR_NONE == ret) {
		memset(g_service_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_service_str, name, MENU_DATA_SIZE);
		g_free(name);
	}
}

static void  ___free_service(gpointer data)
{
	ua_service_h handle = data;
	ua_service_destroy(handle);
	handle = NULL;
}

static void __clear_service_list(void)
{
	g_slist_free_full(g_service_list, ___free_service);
	g_service_list = NULL;
	g_service_count = 0;
}

static bool __foreach_added_service_cb(
	ua_service_h service_handle, void *user_data)
{
	int ret = UA_ERROR_NONE;
	char *name = NULL;
	unsigned int presence_threshold = 0;
	unsigned int absence_threshold = 0;

	if (NULL == service_handle) {
		msgr("service_handle is NULL");
		return true;
	}

	ret = ua_service_get_name(service_handle, &name);
	if (UA_ERROR_NONE != ret) {
		msg(" - ua_service_get_name() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));
		return RET_SUCCESS;
	}

	ret = ua_service_get_detection_threshold(service_handle,
			&presence_threshold, &absence_threshold);
	if (UA_ERROR_NONE != ret) {
		msg(" - ua_service_get_detection_threshold() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));
		g_free(name);
		return RET_SUCCESS;
	}

	msgr("[%d] %s ", ++g_service_count, name);
	msgr("presence threshold :[%d] absence threshold :[%d]",
			presence_threshold, absence_threshold);

	g_service_list = g_slist_append(g_service_list, service_handle);

	g_free(name);

	return true;
}

static bool _service_foreach_added_device_cb(
	ua_service_h service_handle, ua_device_h device_handle, void *user_data)
{
	int ret, ret_temp;
	ua_device_h handle = NULL;
	char *service_name = NULL;

	ua_mac_type_e mac_type = UA_MAC_TYPE_INVALID;
	ua_os_type_e ostype = UA_OS_TYPE_NOT_DEFINE;
	char *device_id = NULL;
	char *mac = NULL;
	char *wifi_bssid = NULL;
	char *ipv4 = NULL;
	bool required = false;
	bool discriminant = false;
	unsigned long long last_seen = 0;

	if (!service_handle) {
		msgr("service_handle is NULL");
		return true;
	}

	ret = ua_service_get_name(service_handle, &service_name);
	if (UA_ERROR_NONE == ret)
		msgb("Cur. Service name : %s", service_name);
	g_free(service_name);

	if (!device_handle) {
		msgr("device_handle is NULL");
		return true;
	}

	ret = ua_device_clone(&handle, device_handle);
	if (UA_ERROR_NONE != ret) {
		msgr(" - ua_device_clone() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));
		return true;
	}

	msgb("\n[%d]", ++g_device_count);

	ret_temp = ua_device_get_mac_type(handle, &mac_type);
	if (UA_ERROR_NONE == ret_temp)
		msgb("Device MAC Type : %s", uat_get_mac_type_str(mac_type));

	ret_temp = ua_device_get_os_info(handle, &ostype);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device OS Type : %s",
			ostype == 1 ? "Tizen" :
			ostype == 2 ? "Android" :
			ostype == 3 ? "iOS" : "N/A");
	}

	ret_temp = ua_device_get_device_id(handle, &device_id);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device ID : %s", device_id);
		g_free(device_id);
	}

	ret_temp = ua_device_get_mac_address(handle, &mac);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device MAC : %s", mac);
		g_free(mac);
	}

	ret_temp = ua_device_get_wifi_bssid(handle, &wifi_bssid);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device BSSID : %s", wifi_bssid);
		g_free(wifi_bssid);
	}

	ret_temp = ua_device_get_wifi_ipv4_address(handle, &ipv4);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device IPv4 Addr : %s", ipv4);
		g_free(ipv4);
	}

	ret_temp = ua_device_get_pairing_required(handle, &required);
	if (UA_ERROR_NONE == ret_temp)
		msgb("Pairing Required : %s", required ? "YES" : "NO");

	ret_temp = ua_device_get_discriminant(handle, &discriminant);
	if (UA_ERROR_NONE == ret_temp)
		msgb("Discriminant : %s", discriminant ? "Enabled" : "Disabled");

	ret_temp = ua_device_get_last_presence(handle, &last_seen);
	if (UA_ERROR_NONE == ret_temp)
		msgb("Last present at : %llu", last_seen);

	g_device_list = g_slist_append(g_device_list, handle);

	return true;
}

static int uat_select_service(MManager *mm, struct menu_data *menu)
{
	GSList *iter = g_service_list;

	int id = 0;
	int selected_id = 0;

	if (strlen(g_selected_service_id))
		selected_id = (unsigned char)strtol(g_selected_service_id, NULL, 10);

	if (selected_id <= 0) {
		msg("Please select service first");
		return RET_SUCCESS;
	}

	while (NULL != iter) {
		GSList *next = iter->next;
		ua_service_h *handle = iter->data;
		if (handle && ++id == selected_id) {
		    /* Make selected service as a current one */
		    g_service_h = handle;
		    __update_service_info();
		    msg("Select service done");
		    return RET_SUCCESS;
		}
		iter = next;
	}

	return RET_SUCCESS;
}

static int run_ua_service_create(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_create");

	if (g_service_h) {
		ua_service_destroy(g_service_h);
		g_service_h = NULL;
	}

	ret = ua_service_create(&g_service_h);

	msg(" - ua_service_create() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_destroy(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_destroy");

	ret = ua_service_destroy(g_service_h);
	g_service_h = NULL;

	msg(" - ua_service_destroy() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_set_name(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_set_name");

	check_if(NULL == g_service_h);

	ret = ua_service_set_name(g_service_h, g_service_str);

	msg(" - ua_service_set_name() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_set_detection_threshold(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	unsigned int presence_threshold = 60;
	unsigned int absence_threshold = 60;

	msg("ua_service_set_threshold");

	check_if(NULL == g_service_h);

	if (strlen(g_presence_threshold_str))
		presence_threshold = (unsigned int)strtol(g_presence_threshold_str, NULL, 10);
	if (strlen(g_absence_threshold_str))
		absence_threshold = (unsigned int)strtol(g_absence_threshold_str, NULL, 10);

	ret = ua_service_set_detection_threshold(g_service_h, presence_threshold,
					absence_threshold);

	msg(" - ua_service_set_name() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_add(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_add");

	check_if(NULL == g_service_h);

	ret = ua_service_add(g_service_h);

	msg(" - ua_service_add() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_update(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_update");

	check_if(NULL == g_service_h);

	ret = ua_service_update(g_service_h);

	msg(" - ua_service_update() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_remove(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_remove");

	check_if(NULL == g_service_h);

	ret = ua_service_remove(g_service_h);

	msg(" - ua_service_remove() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_get_default_service(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_get_default_service");

	if (g_service_h) {
		ua_service_destroy(g_service_h);
		g_service_h = NULL;
	}

	ret = ua_service_get_default_service(&g_service_h);

	if (UA_ERROR_NONE == ret)
		__update_service_info();

	msg(" - ua_service_get_default_service() ret: [0x%X] [%s]",
		ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_get_by_name(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_get_by_name");

	if (g_service_h) {
		ua_service_destroy(g_service_h);
		g_service_h = NULL;
	}

	ret = ua_service_get_by_name(g_service_str, &g_service_h);

	if (UA_ERROR_NONE == ret)
		__update_service_info();

	msg(" - ua_service_get_by_name() ret: [0x%X] [%s]",
		ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_foreach_added(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	msg("ua_service_foreach_added");

	__clear_service_list();

	ret = ua_service_foreach_added(__foreach_added_service_cb, NULL);

	msg(" - ua_service_foreach_added() ret: [0x%X] [%s]",
				ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_add_device(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_add_device");

	check_if(NULL == g_service_h);
	check_if(NULL == g_device_h);

	ret = ua_service_add_device(g_service_h, g_device_h);

	msg(" - ua_service_add_device() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_remove_device(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_remove_device");

	check_if(NULL == g_service_h);
	check_if(NULL == g_device_h);

	ret = ua_service_remove_device(g_service_h, g_device_h);

	msg(" - ua_service_remove_device() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_set_device_discriminant(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	int temp = 1;
	gboolean discriminant = true;

	msg("ua_service_set_device_discriminant");

	check_if(NULL == g_service_h);
	check_if(NULL == g_device_h);

	if (strlen(g_service_device_discriminant))
		temp = (int)strtol(g_service_device_discriminant, NULL, 10);

	if (0 == temp)
		discriminant = false;
	ret = ua_service_set_device_discriminant(g_service_h, g_device_h, discriminant);

	msg(" - ua_service_add_device() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_get_device_discriminant(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	gboolean discriminant = true;

	msg("ua_service_get_device_discriminant");

	check_if(NULL == g_service_h);
	check_if(NULL == g_device_h);

	ret = ua_service_get_device_discriminant(g_service_h, g_device_h, &discriminant);

	msg(" - ua_service_add_device() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	if (UA_ERROR_NONE == ret)
		msgb("Device Discriminant %d", discriminant);

	return RET_SUCCESS;
}

static int run_ua_service_foreach_added_devices(MManager *mm,
	struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	ua_service_h service_handle;

	msg("ua_service_foreach_added_devices");
	ret = ua_service_get_by_name(g_service_str, &service_handle);
	if (UA_ERROR_NONE == ret) {
		msgb("Service Handle [%p]", service_handle);
		ua_service_destroy(g_service_h);
		g_service_h = service_handle;
		__update_service_info();
	} else {
		msgr("ua_service_get_by_name()  ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));
	}

	uat_clear_device_list();

	ret = ua_service_foreach_added_devices(g_service_h,
		_service_foreach_added_device_cb, NULL);

	msg(" - ua_service_foreach_added_devices() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_add_user(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_add_user");

	check_if('\0' == g_user_account_str[0]);
	check_if(NULL == g_user_h);
	check_if('\0' == g_service_str[0]);
	check_if(NULL == g_service_h);

	ret = ua_service_add_user(g_service_h, g_user_h);

	msg(" - ua_service_add_user() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_remove_user(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_remove_user");

	check_if('\0' == g_user_account_str[0]);
	check_if(NULL == g_user_h);
	check_if('\0' == g_service_str[0]);
	check_if(NULL == g_service_h);

	ret = ua_service_remove_user(g_service_h, g_user_h);

	msg(" - ua_service_remove_user() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}


static int run_ua_service_foreach_added_users(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	msg("ua_service_foreach_added_users");

	check_if(NULL == g_service_h);

	uat_clear_user_list();

	ret = ua_service_foreach_added_users(g_service_h,
				_service_foreach_added_user_cb, NULL);

	msg(" - ua_service_foreach_added_users() ret: [0x%X] [%s]",
				ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static struct menu_data menu_ua_service_set_name[] = {
	{ "1", "name", NULL,
		NULL, g_service_str },
	{ "2", "run", NULL,
		run_ua_service_set_name, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_service_get_by_name[] = {
	{ "1", "name", NULL,
		NULL, g_service_str },
	{ "2", "run", NULL,
		run_ua_service_get_by_name, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_service_set_detection_threshold[] = {
	{ "1", "light presence detection threshold",
		NULL, NULL, g_presence_threshold_str},
	{ "2", "light absence detection threshold",
		NULL, NULL, g_absence_threshold_str},
	{ "3", "run",
		NULL, run_ua_service_set_detection_threshold, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_service_set_device_discriminant[] = {
	{ "1", "discriminant (0:disable 1:enable)",
		NULL, NULL, g_service_device_discriminant },
	{ "2", "run", NULL,
		run_ua_service_set_device_discriminant, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_sel_service[] = {
	{ "1", "Service list", NULL,
		run_ua_service_foreach_added, g_selected_service_id},
	{ "2", "Apply", NULL, uat_select_service, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_service_add_user[] = {
	{ "-", "service",
		NULL, NULL, g_service_str},
	{ "-", "user account",
		NULL, NULL, g_user_account_str},
	{ "-", "user name",
		NULL, NULL, g_user_name_str},
	{ "1", "run", NULL,
		run_ua_service_add_user, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_service_remove_user[] = {
	{ "-", "service",
		NULL, NULL, g_service_str},
	{ "-", "user account",
		NULL, NULL, g_user_account_str},
	{ "-", "user name",
		NULL, NULL, g_user_name_str},
	{ "1", "run", NULL,
		run_ua_service_remove_user, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_sel_service_added_device[] = {
	{ "1", "Device list", NULL,
		run_ua_service_foreach_added_devices, g_selected_device_idx},
	{ "2", "Apply", NULL, uat_select_device, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_sel_service_added_user[] = {
	{ "1", "User list", NULL,
		run_ua_service_foreach_added_users, g_selected_user_idx},
	{ "2", "Apply", NULL, uat_select_user, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_ua_services[] = {
	{ "1", "ua_service_create",
		NULL, run_ua_service_create, NULL },
	{ "2", "ua_service_set_name",
		menu_ua_service_set_name, NULL, g_service_str },
	{ "3", "ua_service_set_detection_threshold",
		menu_ua_service_set_detection_threshold, NULL, NULL },
	{ "4", "ua_service_add",
		NULL, run_ua_service_add, NULL },
	{ "5", "ua_service_update",
		NULL, run_ua_service_update, NULL },
	{ "6", "ua_service_remove",
		NULL, run_ua_service_remove, NULL},
	{ "7", "ua_service_destroy",
		NULL, run_ua_service_destroy, NULL},
	{ "8", "ua_service_get_default_service",
		NULL, run_ua_service_get_default_service, NULL },
	{ "9", "ua_service_get_by_name",
		menu_ua_service_get_by_name, NULL, NULL },
	{ "10", ANSI_COLOR_LIGHTMAGENTA "ua_service_foreach_added" ANSI_COLOR_NORMAL,
		menu_sel_service, NULL, NULL },
	{ "21", ANSI_COLOR_LIGHTGREEN "ua_service_add_device" ANSI_COLOR_NORMAL,
		NULL, run_ua_service_add_device, NULL },
	{ "22", ANSI_COLOR_LIGHTRED "ua_service_remove_device" ANSI_COLOR_NORMAL,
		NULL, run_ua_service_remove_device, NULL },
	{ "23", "ua_service_set_device_discriminant",
		menu_ua_service_set_device_discriminant, NULL, NULL },
	{ "24", "ua_service_get_device_discriminant",
		NULL, run_ua_service_get_device_discriminant, NULL },
	{ "25", ANSI_COLOR_LIGHTMAGENTA "ua_service_foreach_added_devices" ANSI_COLOR_NORMAL,
		menu_sel_service_added_device, NULL, NULL },
	{ "31", ANSI_COLOR_LIGHTGREEN "ua_service_add_user" ANSI_COLOR_NORMAL,
		menu_ua_service_add_user, NULL, NULL },
	{ "32", ANSI_COLOR_LIGHTRED "ua_service_remove_user" ANSI_COLOR_NORMAL,
		menu_ua_service_remove_user, NULL, NULL },
	{ "33", ANSI_COLOR_LIGHTMAGENTA "ua_service_foreach_added_users" ANSI_COLOR_NORMAL,
		menu_sel_service_added_user, NULL, NULL },
	{ NULL, NULL, },
};
